#!/usr/bin/python

import unittest

from mockssoservice import mockserver
from wsgi_intercept import add_wsgi_intercept, remove_wsgi_intercept
from wsgi_intercept.httplib2_intercept import install, uninstall
from piston_mini_client.auth import BasicAuthorizer, OAuthAuthorizer
from piston_mini_client.failhandlers import APIError
from ssoclient import SingleSignOnAPI

class SSOTestCase(unittest.TestCase):
    def mock_sso(self, *args):
        return mockserver.MockSSOServer('login.ubuntu.com', 443, '/api/1.0',
                             scheme='https')

    def setUp(self):
        install()
        self.service_root = 'https://login.ubuntu.com/api/1.0'
        self.basic_server_auth = BasicAuthorizer(username='MyUsername',
            password='password')
        self.basic_user_auth = BasicAuthorizer(username='mail@somedomain.com',
            password='OMGTh1sIs5trong')
        self.oauth_auth=OAuthAuthorizer('tokenkey', 'tokensecret',
            'consumerkey', 'consumersecret')
        add_wsgi_intercept('login.ubuntu.com', 443, self.mock_sso)
        super(SSOTestCase, self).setUp()

    def tearDown(self):
        super(SSOTestCase, self).tearDown()
        remove_wsgi_intercept('login.ubuntu.com', 443)
        uninstall()


class CaptchaTestCase(SSOTestCase):
    def test_new_captcha(self):
        api = SingleSignOnAPI(self.service_root)

        captcha = api.new_captcha()

        expected = set(['captcha_id', 'image_url'])
        self.assertEqual(expected, set(captcha.keys()))


class RegisterTestCase(SSOTestCase):
    def test_register_weak_password(self):
        api = SingleSignOnAPI(self.service_root)
        kwargs = {'email': 'foo@bar.baz',
            'captcha_id': 'someid',
            'captcha_solution': 'omg lol',
            }
        weak_passwords = [
            'm3D*',
            'onlylower',
            'ONLYUPPER',
            'honestlyihavenoideawhythisisstilltooweak',
            'AlmostThere',
            ]
        for password in weak_passwords:
            r = api.register(password=password, **kwargs)
            self.assertEqual('error', r['status'])
            self.assertEqual(['password'], r['errors'].keys())

    def test_invalid_email(self):
        api = SingleSignOnAPI(self.service_root)
        r = api.register(password='H1l4riuos',
            captcha_id='someid',
            captcha_solution='omg lol',
            email='invalid')
        self.assertEqual('error', r['status'])
        self.assertEqual(['email'], r['errors'].keys())


class RequestPasswordResetTokenTestCase(SSOTestCase):
    def test_request_password_reset_token(self):
        api = SingleSignOnAPI(self.service_root)
        result = api.request_password_reset_token('myemail@test.com')
        expected = {'status': 'ok', 'message': 'Password reset token sent.'}
        self.assertEqual(expected, result)


class SetNewPasswordTestCase(SSOTestCase):
    def test_set_new_password_invalid_password(self):
        api = SingleSignOnAPI(self.service_root)
        invalid_passwords = [
            'Shor7',
            'onlylowercase',
            'etc...',
        ]
        expected = {
            'status': 'error',
            'errors': [
                'Password must be at least 8 characters long, and must '
                'contain at least one number and an upper case letter.'
            ],
        }
        for passwd in invalid_passwords:
            result = api.set_new_password('foobar@baz.com', passwd, 'token')
            self.assertEqual(expected, result)

    def test_set_new_password_success(self):
        api = SingleSignOnAPI(self.service_root)
        result = api.set_new_password('foobar@baz.com', 'Q1w2E3r4', 'token')
        expected = {'message': 'Password changed', 'status': 'ok'}
        self.assertEqual(expected, result)


class AuthenticateTestCase(SSOTestCase):
    def test_authenticate_failure(self):
        # Attempt to authenticate an APIUser fails using the mock server
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        self.assertRaises(APIError, api.authenticate, token_name='foobar')

    def test_authenticate_success(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_user_auth)
        result = api.authenticate(token_name='foobar')
        expected = set(['consumer_secret', 'token', 'consumer_key', 'name',
            'token_secret'])
        self.assertEqual(expected, set(result.keys()))


class ListTokensTestCase(SSOTestCase):
    def test_list_tokens_auth_failure(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_user_auth)
        self.assertRaises(APIError, api.list_tokens, 'openid-123')

    def test_list_tokens_no_tokens(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        self.assertEqual([], api.list_tokens('openid-123'))

    def test_list_tokens_some_tokens(self):
        mockserver.tokens.clear()
        mockserver.new_token('foobar')
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        self.assertEqual(1, len(api.list_tokens('name12_oid')))


class ValidateTokenTestCase(SSOTestCase):
    def test_validate_token_auth_faliure(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_user_auth)
        self.assertRaises(APIError, api.validate_token, 'openid-123', 'token')

    def test_validate_token_success(self):
        mockserver.tokens.clear()
        token = '0123456789abcdef'
        mockserver.new_token('token-name', token=token)
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        expected = set(['consumer_secret', 'token', 'consumer_key', 'name',
            'token_secret'])

        result = api.validate_token(consumer_key='name12_oid', token=token)

        self.assertEqual(expected, set(result.keys()))


class InvalidateTokenTestCase(SSOTestCase):
    def test_invalidate_token_auth_faliure(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_user_auth)
        self.assertRaises(APIError, api.invalidate_token, 'someoid', 'token')

    def test_invalidate_token_success(self):
        mockserver.tokens.clear()
        token = '0123456789abcdef'
        mockserver.new_token('token-name', token=token)
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        result = api.invalidate_token(consumer_key='name12_oid', token=token)
        self.assertEqual(None, result)


class TeamMembershipsByOpenid(SSOTestCase):
    def test_team_memberships_by_openid_auth_failure(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_user_auth)
        self.assertRaises(APIError, api.invalidate_token, 'someoid', 'token')

    def test_team_memberships_by_openid_success(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)
        result = api.team_memberships_by_openid('someoid',
            ["ubuntu-team", "myteam", "someotherteam"])

        self.assertEqual(["ubuntu-team", "myteam"], result)


class AccountByEmailTestCase(SSOTestCase):
    def test_account_by_email_invalid_email(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)

        result = api.account_by_email(email='someemail@bla.com')
        self.assertEqual(None, result)

    def test_account_by_email_verified(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)

        result = api.account_by_email(email='blu@bli.com')
        self.assertEqual('username', result.username)
        self.assertEqual('Blu Bli', result.displayname)
        self.assertEqual('blu@bli.com', result.preferred_email)
        self.assertEqual([], result.verified_emails)
        self.assertEqual([], result.unverified_emails)

class AccountByOpenIDTestCase(SSOTestCase):
    def test_account_by_openid_invalid_identifier(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)

        result = api.account_by_openid(openid='oid_1234')
        self.assertEqual(None, result)

    def test_account_by_openid_success(self):
        api = SingleSignOnAPI(self.service_root, auth=self.basic_server_auth)

        result = api.account_by_openid(openid='openid_identifier')
        self.assertEqual('username', result.username)
        self.assertEqual('Blu Bli', result.displayname)
        self.assertEqual('openid_identifier', result.openid_identifier)


class MeTestCase(SSOTestCase):
    def test_me(self):
        api = SingleSignOnAPI(self.service_root, auth=self.oauth_auth)
        result = api.me()
        self.assertEqual('Blu Bli', result.displayname)


class MyTeamMembershipsTestcase(SSOTestCase):
    def test_my_team_memberships(self):
        api = SingleSignOnAPI(self.service_root, auth=self.oauth_auth)
        result = api.my_team_memberships(['foobar'])
        self.assertEqual([], result)


class ValidateEmailTestCase(SSOTestCase):
    def test_validate_email_fail(self):
        api = SingleSignOnAPI(self.service_root, auth=self.oauth_auth)
        result = api.validate_email('abcdefghijkl1234567890')
        expected = {'errors': {'email_token': ['Bad email token!']}}
        self.assertEqual(expected, result)

    def test_validate_email_success(self):
        api = SingleSignOnAPI(self.service_root, auth=self.oauth_auth)
        result = api.validate_email('jJRkmngbHjmnJDEK')
        self.assertEqual({'email': 'blu@bli.com'}, result)


if __name__ == "__main__":
    unittest.main()
